@@ -54,7 +54,7 @@
 #define PCAP_DONT_INCLUDE_PCAP_BPF_H 1 /* don't re-include it in pcap.h */
 #endif
 
-#include "/usr/include/pcap.h"
+#include "/Applications/Xcode_14.0.1.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/pcap.h"
 
 /* include our own strlcat/strlcpy? */
 #ifndef HAVE_STRLCPY
