@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -31,7 +31,11 @@ int dlt_raw_decode(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 int dlt_raw_encode(tcpeditdlt_t *ctx, u_char *packet, int pktlen, tcpr_dir_t dir);
 int dlt_raw_proto(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 u_char *dlt_raw_get_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen);
-u_char *dlt_raw_merge_layer3(tcpeditdlt_t *ctx, u_char *packet, const int pktlen, u_char *l3data);
+u_char *dlt_raw_merge_layer3(tcpeditdlt_t *ctx,
+                             u_char *packet,
+                             const int pktlen,
+                             u_char *ipv4_data,
+                             u_char *ipv6_data);
 tcpeditdlt_l2addr_type_t dlt_raw_l2addr_type(void);
 int dlt_raw_l2len(tcpeditdlt_t *ctx, const u_char *packet, const int pktlen);
 u_char *dlt_raw_get_mac(tcpeditdlt_t *ctx, tcpeditdlt_mac_type_t mac, const u_char *packet, const int pktlen);
