@@ -69,6 +69,9 @@ typedef struct {
     int dlt2;
     char errstr[TCPEDIT_ERRSTR_LEN];
     char warnstr[TCPEDIT_ERRSTR_LEN];
+#ifdef FORCE_ALIGN    
+    u_char *l3buff;
+#endif
 } tcpedit_runtime_t;
 
 /*
